@@ -12,6 +12,7 @@
 
 import contextlib
 import io
+import json
 import sys
 import os
 import re
@@ -25,34 +26,33 @@ import pipes
 import platform
 import zipfile
 import shutil
+import sysconfig
 import glob
 
 try:  # pragma: no cover (py2 only)
     from ConfigParser import SafeConfigParser as ConfigParser
     # noinspection PyCompatibility
-    from HTMLParser import HTMLParser
     import urllib2
     iteritems = operator.methodcaller('iteritems')
+    import httplib
+    IncompleteRead = httplib.IncompleteRead
 except ImportError:  # pragma: no cover (py3 only)
     from configparser import ConfigParser
     # noinspection PyUnresolvedReferences
-    from html.parser import HTMLParser
     import urllib.request as urllib2
     iteritems = operator.methodcaller('items')
+    import http
+    IncompleteRead = http.client.IncompleteRead
 
 from pkg_resources import parse_version
 
-nodeenv_version = '1.3.5'
+nodeenv_version = '1.4.0'
 
 join = os.path.join
 abspath = os.path.abspath
-iojs_taken = False
-src_domain = "nodejs.org"
+src_base_url = None
 
 is_PY3 = sys.version_info[0] >= 3
-if is_PY3:
-    from functools import cmp_to_key
-
 is_WIN = platform.system() == 'Windows'
 is_CYGWIN = platform.system().startswith('CYGWIN')
 
@@ -225,15 +225,10 @@ def parse_args(check=True):
         'The default is last stable version (`latest`). '
         'Use `system` to use system-wide node.')
 
-    parser.add_option(
-        '-i', '--iojs',
-        action='store_true', dest='io', default=False,
-        help='Use iojs instead of nodejs.')
-
     parser.add_option(
         '--mirror',
         action="store", dest='mirror',
-        help='Set mirror server of nodejs.org or iojs.org to download from.')
+        help='Set mirror server of nodejs.org to download from.')
 
     if not is_WIN:
         parser.add_option(
@@ -500,9 +495,13 @@ def callit(cmd, show_stdout=True, in_shell=False,
 
 def get_root_url(version):
     if parse_version(version) > parse_version("0.5.0"):
-        return 'https://%s/dist/v%s/' % (src_domain, version)
+        return '%s/v%s/' % (src_base_url, version)
     else:
-        return 'https://%s/dist/' % src_domain
+        return src_base_url
+
+
+def is_x86_64_musl():
+    return sysconfig.get_config_var('HOST_GNU_TYPE') == 'x86_64-pc-linux-musl'
 
 
 def get_node_bin_url(version):
@@ -522,15 +521,16 @@ def get_node_bin_url(version):
     }
     if is_WIN or is_CYGWIN:
         postfix = '-win-%(arch)s.zip' % sysinfo
-        filename = '%s-v%s%s' % (get_binary_prefix(), version, postfix)
+    elif is_x86_64_musl():
+        postfix = '-linux-x64-musl.tar.gz'
     else:
         postfix = '-%(system)s-%(arch)s.tar.gz' % sysinfo
-        filename = '%s-v%s%s' % (get_binary_prefix(), version, postfix)
+    filename = 'node-v%s%s' % (version, postfix)
     return get_root_url(version) + filename
 
 
 def get_node_src_url(version):
-    tar_name = '%s-v%s.tar.gz' % (get_binary_prefix(), version)
+    tar_name = 'node-v%s.tar.gz' % version
     return get_root_url(version) + tar_name
 
 
@@ -544,12 +544,17 @@ def tarfile_open(*args, **kwargs):
         tf.close()
 
 
-def download_node_src(node_url, src_dir, opt, prefix):
+def download_node_src(node_url, src_dir, opt):
     """
     Download source code
     """
     logger.info('.', extra=dict(continued=True))
-    dl_contents = io.BytesIO(urlopen(node_url).read())
+    try:
+        dl_contents = io.BytesIO(urlopen(node_url).read())
+    except IncompleteRead as e:
+        logger.warning('Incomplete read while reading'
+                       'from {}'.format(node_url))
+        dl_contents = e.partial
     logger.info('.', extra=dict(continued=True))
 
     if is_WIN or is_CYGWIN:
@@ -563,8 +568,8 @@ def download_node_src(node_url, src_dir, opt, prefix):
 
     with ctx as archive:
         node_ver = re.escape(opt.node)
-        rexp_string = r"%s-v%s[^/]*/(README\.md|CHANGELOG\.md|LICENSE)"\
-            % (prefix, node_ver)
+        rexp_string = r"node-v%s[^/]*/(README\.md|CHANGELOG\.md|LICENSE)"\
+            % node_ver
         extract_list = [
             member
             for member in members(archive)
@@ -606,7 +611,6 @@ def copy_node_from_prebuilt(env_dir, src_dir, node_version):
     Copy prebuilt binaries into environment
     """
     logger.info('.', extra=dict(continued=True))
-    prefix = get_binary_prefix()
     if is_WIN:
         dest = join(env_dir, 'Scripts')
         mkdir(dest)
@@ -618,7 +622,7 @@ def copy_node_from_prebuilt(env_dir, src_dir, node_version):
     else:
         dest = env_dir
 
-    src_folder_tpl = src_dir + to_utf8('/%s-v%s*' % (prefix, node_version))
+    src_folder_tpl = src_dir + to_utf8('/node-v%s*' % node_version)
     src_folder, = glob.glob(src_folder_tpl)
     copytree(src_folder, dest, True)
 
@@ -687,10 +691,6 @@ def build_node_from_src(env_dir, src_dir, node_src_dir, opt):
     callit([make_cmd + ' install'], opt.verbose, True, node_src_dir, env)
 
 
-def get_binary_prefix():
-    return to_utf8('node' if not iojs_taken else 'iojs')
-
-
 def install_node(env_dir, src_dir, opt):
     """
     Download source code for node.js, unpack it
@@ -706,11 +706,10 @@ def install_node(env_dir, src_dir, opt):
 
 def install_node_wrapped(env_dir, src_dir, opt):
     env_dir = abspath(env_dir)
-    prefix = get_binary_prefix()
-    node_src_dir = join(src_dir, to_utf8('%s-v%s' % (prefix, opt.node)))
+    node_src_dir = join(src_dir, to_utf8('node-v%s' % opt.node))
     src_type = "prebuilt" if opt.prebuilt else "source"
 
-    logger.info(' * Install %s %s (%s) ' % (src_type, prefix, opt.node),
+    logger.info(' * Install %s node (%s) ' % (src_type, opt.node),
                 extra=dict(continued=True))
 
     if opt.prebuilt:
@@ -720,7 +719,7 @@ def install_node_wrapped(env_dir, src_dir, opt):
 
     # get src if not downloaded yet
     if not os.path.exists(node_src_dir):
-        download_node_src(node_url, src_dir, opt, prefix)
+        download_node_src(node_url, src_dir, opt)
 
     logger.info('.', extra=dict(continued=True))
 
@@ -954,59 +953,13 @@ def create_environment(env_dir, opt):
         shutil.rmtree(src_dir)
 
 
-class GetsAHrefs(HTMLParser):
-    def __init__(self):
-        # Old style class in py2 :(
-        HTMLParser.__init__(self)
-        self.hrefs = []
-
-    def handle_starttag(self, tag, attrs):
-        if tag == 'a':
-            self.hrefs.append(dict(attrs).get('href', ''))
-
-
-VERSION_RE = re.compile(r'\d+\.\d+\.\d+')
-
-
-def _py2_cmp(a, b):
-    # -1 = a < b, 0 = eq, 1 = a > b
-    return (a > b) - (a < b)
-
-
-def compare_versions(version, other_version):
-    version_tuple = version.split('.')
-    other_tuple = other_version.split('.')
-
-    version_length = len(version_tuple)
-    other_length = len(other_tuple)
-    version_dots = min(version_length, other_length)
-
-    for i in range(version_dots):
-        a = int(version_tuple[i])
-        b = int(other_tuple[i])
-        cmp_value = _py2_cmp(a, b)
-        if cmp_value != 0:
-            return cmp_value
-
-    return _py2_cmp(version_length, other_length)
+def _get_versions_json():
+    response = urlopen('%s/index.json' % src_base_url)
+    return json.loads(response.read().decode('UTF-8'))
 
 
 def get_node_versions():
-    response = urlopen('https://{0}/dist'.format(src_domain))
-    href_parser = GetsAHrefs()
-    href_parser.feed(response.read().decode('UTF-8'))
-
-    versions = set(
-        VERSION_RE.search(href).group()
-        for href in href_parser.hrefs
-        if VERSION_RE.search(href)
-    )
-    if is_PY3:
-        key_compare = cmp_to_key(compare_versions)
-        versions = sorted(versions, key=key_compare)
-    else:
-        versions = sorted(versions, cmp=compare_versions)
-    return versions
+    return [dct['version'].lstrip('v') for dct in _get_versions_json()][::-1]
 
 
 def print_node_versions():
@@ -1025,23 +978,7 @@ def get_last_stable_node_version():
     """
     Return last stable node.js version
     """
-    response = urlopen('https://%s/dist/latest/' % src_domain)
-    href_parser = GetsAHrefs()
-    href_parser.feed(response.read().decode('UTF-8'))
-
-    links = []
-    pattern = re.compile(r'''%s-v([0-9]+)\.([0-9]+)\.([0-9]+)\.tar\.gz''' % (
-        get_binary_prefix()))
-
-    for href in href_parser.hrefs:
-        match = pattern.match(href)
-        if match:
-            version = u'.'.join(match.groups())
-            major, minor, revision = map(int, match.groups())
-            links.append((version, major, minor, revision))
-            break
-
-    return links[-1][0]
+    return _get_versions_json()[0]['version'].lstrip('v')
 
 
 def get_env_dir(opt, args):
@@ -1060,16 +997,6 @@ def get_env_dir(opt, args):
     return to_utf8(res)
 
 
-def is_installed(name):
-    try:
-        devnull = open(os.devnull)
-        subprocess.Popen([name], stdout=devnull, stderr=devnull)
-    except OSError as e:
-        if e.errno == os.errno.ENOENT:
-            return False
-    return True
-
-
 # noinspection PyProtectedMember
 def main():
     """
@@ -1090,15 +1017,21 @@ def main():
         logger.error('Installing system node.js on win32 is not supported!')
         exit(1)
 
-    global iojs_taken
-    global src_domain
-
-    if opt.io:
-        iojs_taken = True
-        src_domain = "iojs.org"
+    global src_base_url
 
+    src_domain = None
     if opt.mirror:
-        src_domain = opt.mirror
+        if '://' in opt.mirror:
+            src_base_url = opt.mirror
+        else:
+            src_domain = opt.mirror
+    # use unofficial builds only if musl and no explicitly chosen mirror
+    elif is_x86_64_musl():
+        src_domain = 'unofficial-builds.nodejs.org'
+    else:
+        src_domain = 'nodejs.org'
+    if src_base_url is None:
+        src_base_url = 'https://%s/download/release' % src_domain
 
     if not opt.node or opt.node.lower() == "latest":
         opt.node = get_last_stable_node_version()
@@ -1413,6 +1346,8 @@ function deactivate_node -d 'Exit nodeenv and return to normal environment.'
         set -e _OLD_NODE_FISH_PROMPT_OVERRIDE
     end
 
+    set -e NODE_VIRTUAL_ENV
+
     if test (count $argv) = 0 -o "$argv[1]" != "nondestructive"
         # Self destruct!
         functions -e deactivate_node
